/*
laserspotcam        A software to track a laser pointer on a wall.
Copyright (C) 2005  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "SDL.h"

#include "SDL_v4l.h"
#include "webcam.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

// RAPH 2005: Hacked for grayscale and removed uses of ccvt

/* Video for linux and philips webcam includes */
//#include <linux/videodev.h>
//#include <sys/ioctl.h>
//#include "pwc-ioctl.h"

/* Color space conversion routines */
//#include "ccvt/ccvt.h"

#define MAX_VIDEO_IMAGE_SIZE 460800 // 640x480x1.5 YUV4:2:0P
#define MAX_VIDEO_Y_BUFFER_SIZE 307200 // 640x480 Y component
#define MAX_VIDEO_U_BUFFER_SIZE 76800 // 640x480x0.25 U component
#define MAX_VIDEO_V_BUFFER_SIZE 76800 // 640x480x0.25 V component
#define MAX_VIDEO_RGB_BUFFER_SIZE 1228800 // 640x480x4  32 Bit RGB

extern SDL_Surface *screen;
static int g_webcam_descriptor;
static int g_webcam_capture_width;
static int g_webcam_capture_height;
static int g_rotation=0;  // If TRUE, 90degrees
	
static unsigned char RGB_buffer[MAX_VIDEO_RGB_BUFFER_SIZE];

char SDL_v4l_setRotation(char rotation)
{
	g_rotation = rotation;
	return g_rotation;
}

char SDL_v4l_toggleRotation()
{
	g_rotation = !(g_rotation);
	return g_rotation;
}

char SDL_v4l_getRotation()
{
	return g_rotation;
}

int SDL_v4l_getRGB32Data(unsigned char *data, unsigned long len)
{
	/* This only works on webcams supporting YUV4:2:0P
	 * I cannot test other palettes since my webcam only supports
	 * YUV4:2:0P. */
	int webcam_y_component_size;
	unsigned char Y_buffer[MAX_VIDEO_Y_BUFFER_SIZE];
	unsigned char U_buffer[MAX_VIDEO_U_BUFFER_SIZE];
	unsigned char V_buffer[MAX_VIDEO_V_BUFFER_SIZE];

	/* Read the raw data */
	get_webcam_resolution(&g_webcam_capture_width, &g_webcam_capture_height);
	g_webcam_descriptor = get_webcam_filedescriptor();
	webcam_y_component_size = g_webcam_capture_width*g_webcam_capture_height;
	read(g_webcam_descriptor, Y_buffer, webcam_y_component_size) ;      // Read the Y plane
	read(g_webcam_descriptor, U_buffer, webcam_y_component_size/4);      // Read the U plane
	read(g_webcam_descriptor, V_buffer, webcam_y_component_size/4);      // Read the V plane

//	ccvt_420p_bgr32(g_webcam_capture_width, g_webcam_capture_height, Y_buffer, U_buffer, V_buffer, data);
	memcpy(data, Y_buffer, webcam_y_component_size);
	
	return 0;
}

int SDL_v4l_get_webcam_image_geometry(int *w, int *h)
{
	if (!g_rotation) {
		get_webcam_resolution(w, h);
	} else 	{
		get_webcam_resolution(h, w);
	}
	return 0;
}

/**
 * Get an image from the webcam and convert it to grayscale.
 * Image is accessible in rgb_buffer..
 */
int SDL_v4l_fetchImage()
{
	/* Get a converted RGB image from the video source */
	return SDL_v4l_getRGB32Data(RGB_buffer, MAX_VIDEO_RGB_BUFFER_SIZE);
}

/**
 * Get a pointer to the buffer containing the last image
 * fetched from the webcam.
 */
unsigned char *SDL_v4l_get_rgb_buffer()
{
	return RGB_buffer;
}


// RAPH 2005: Comment: Bad. Works only on 32 bit surfaces :(
/**
 * Draw the last fetched image on an SDL surface.
 *
 * Prerequisite: SDL_v4l_fetchImage has been called.
 */
int SDL_v4l_blitBuffer(int X, int Y, SDL_Surface *target)
{
	int y, i=0;


	/*********** DRAW THE VIDEO *****************/

	/* Lock the screen for direct access to the pixels */
    if ( SDL_MUSTLOCK(screen) ) {
        if ( SDL_LockSurface(screen) < 0 ) {
            fprintf(stderr, "Can't lock screen: %s\n", SDL_GetError());
            return -1;
        }
    }

	i=0;
	for (y=0; y<g_webcam_capture_height; y++) {
		memcpy(target->pixels+(y*target->pitch)+(X*target->format->BytesPerPixel),
		     &RGB_buffer[y*g_webcam_capture_width*4],
			 g_webcam_capture_width*4);
	}

    if ( SDL_MUSTLOCK(screen) ) {
        SDL_UnlockSurface(screen);
    }

	return 0;
}

/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}
