/*
laserspotcam        A software to track a laser pointer on a wall.
Copyright (C) 2005  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "avgdiff.h"

#define NUM_BUFFERS			8

static unsigned char *buffers[NUM_BUFFERS];
static int cur_buffer = 0; // points to the next to be written
static int buffer_new = 1;

static unsigned char *average=NULL;

//static unsigned short tiles[10][7];

static void doRoundRobin(unsigned char *buf, int w, int h);


int AvgDiff_init(int w, int h)
{
	int i;

	memset(buffers, 0, sizeof buffers);
	
	// allocate round robin buffers
	for (i=0; i<NUM_BUFFERS; i++) {
		buffers[i] = malloc(w*h*4);
		if (!buffers) {
			perror("malloc");
			AvgDiff_shutdown();
			return -1;
		}
	}
	
	average = malloc(w*h*4);
	if (!average) {
		perror("malloc");
		AvgDiff_shutdown();
		return -1;
	}

	return 0;
}

void AvgDiff_shutdown(void)
{
	int i;

	for (i=0; i<NUM_BUFFERS; i++) {
		if (buffers[i]) {
			free(buffers[i]);
			buffers[i]=NULL;
		}
	}

	if (average) {
		free(average);
		average = NULL;
	}
}

unsigned char *AvgDiff_getAverageBuffer()
{
	return average;
}

void AvgDiff_process(unsigned char *buf, int w, int h)
{
	int i, len;

	len = w*h;

#if 0
	// convert to grayscale. 
	// TODO: calculate the HUE and map it to grayscale by
	// selecting a window in the red hue zone.
	for (i=0; i<len; i+=4) {
		buf[i]=(buf[i]+buf[i+1]+buf[i+2])/3;
	}
#endif
	// this touches the red channel only
	doRoundRobin(buf, w, h);

	for (i = 0; i<len; i++) {
		buf[i] = buf[i]>average[i] ? buf[i]-average[i] : average[i]-buf[i];
	}
	
}

/**
 * Add a new buffer and recalculate the average
 */
static void doRoundRobin(unsigned char *buf, int w, int h)
{
	int b, i, len;
	int val;

	len = w * h;

	// on first frame, fill all the buffers
	if (buffer_new) {
		for (b = 0; b<NUM_BUFFERS; b++) {
			memcpy(buffers[b], buf, len);
		}
		buffer_new = 0;
	}
	else {	
		// replace the oldest picture
		memcpy(buffers[cur_buffer], buf, len);
		cur_buffer++;
	}

	if (cur_buffer >= NUM_BUFFERS)
		cur_buffer = 0;

	for (i=0; i<len; i++) {
		val = 0;
		for (b=0; b<NUM_BUFFERS; b++) {
			val += buffers[b][i];
		}			
		average[i]= val/NUM_BUFFERS;
	}
}

