/*
laserspotcam        A software to track a laser pointer on a wall.
Copyright (C) 2005  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "daemonize.h"

/* written by Raphael Assenat <raph@raphnet.net>, 
 * using the Unix Programming FAQ, found at
 * http://www.erlenstar.demon.co.uk/unix/faq_toc.html */

/* returns 0 to the parent, 1 to the daemon */
int daemonize(void)
{
	int res;

	/* For so the parent can exit, required to garanteed that
	 * the new process is not a group leader. */
	res = fork();
	if (res<0) {
		perror("First fork failed");
		return res;
	}
	if (res>0) { // parent
		return 0;
	}

	/* Become a process group and session group leader. No controlling
	 * terminal assiciated with this session. */
	res = setsid();
	if (res<0) {
		perror("setsid faild");
		return res;
	}
		
	/* Fork ahain so the parent can exit, and we become a non-session
	 * group leader, and cannot regain a controlling terminal */	
	res = fork();
	if (res<0) {
		perror("Second fork failed");
		return res;
	}
	if (res>0) { // parent
		exit(0);
	}

	/* Chdir to the root directory, to ensure that the process doesn't
	 * keep any directory in use. */
	res = chdir("/");
	if (res<0) {
		perror("chdir(\"/\") failed");
		return res;
	}
	
	/* rw-r--r-- */
	umask(644);

	/* redirect stdin, stdout and stderr to /dev/null */
	open("/dev/null", O_RDWR);
	dup(0);
	dup(0);	
	close(0);
	close(1);
	close(2);

	return 1;
}

