/*
laserspotcam        A software to track a laser pointer on a wall.
Copyright (C) 2005  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hotspots.h"

static hotspot *g_hotspots=NULL;
static int verbose_system=0;

void hotspot_setVerboseSystem(int verb)
{
	verbose_system = verb;
}

hotspot *hotspot_create(int type, int x1, int y1, int x2, int y2, const char *cmd)
{
	hotspot *tmp;

	tmp = malloc(sizeof(hotspot));
	if (!tmp) { 
		perror("malloc");
		return NULL;
	}

	tmp->type = type;
	tmp->x1 = x1;
	tmp->y1 = y1;
	tmp->x2 = x2;
	tmp->y2 = y2;
	tmp->cmd = strdup(cmd);

	return tmp;
}

int hotspot_trig(int test_mode, hotspot *hspot, int x, int y)
{
	struct timeval tv_now;
	int millidiff;
	char cmdbuf[1024];
	int pc;

	gettimeofday(&tv_now, NULL);

	millidiff = ((tv_now.tv_sec*1000)+(tv_now.tv_usec/1000)) -
				(	(hspot->last_triggered.tv_sec*1000) +
					(hspot->last_triggered.tv_usec/1000));

	memcpy(&hspot->last_triggered, &tv_now, sizeof(hspot));
	if (millidiff>1000) {
		if (hspot->type == TYPE_LEFT_TO_RIGHT_VALUE) {
			if (x>hspot->x2) { x = hspot->x2; }
			if (x<hspot->x1) { x = hspot->x1; }

			x -= hspot->x1;
			
			pc = x * 100 / (hspot->x2-hspot->x1 + 1);
				
			snprintf(cmdbuf, 1024, "%s %d", hspot->cmd, pc);
			if (verbose_system) {
				printf("Executing '%s'\n", cmdbuf);
			}
			
			if (!test_mode)
				system(hspot->cmd);
		}
		else {
			if (verbose_system) {
				printf("Executing '%s'\n", hspot->cmd);
			}
			if (!test_mode)
				system(hspot->cmd);	
		}
		return 1;
	}
	return 0;
}


void hotspot_free(hotspot *hspot)
{
	if (hspot) {
		if (hspot->cmd) {
			free(hspot->cmd);
		}
		free(hspot);
	}
}

int hotspot_add(hotspot *hspot)
{
	hotspot *t;

	if (!hspot) { return -1; }
	
	if (g_hotspots==NULL) {
		g_hotspots = hspot;
	} else {
		t = g_hotspots;
		while (t->next) {
			t = t->next;
		}
		t->next = hspot;
	}

	return 0;
}

int hotspot_load(char *filename)
{
	FILE *fptr;
	char linebuf[2048];
	char cmdbuf[2048];
	int x1, y1, x2, y2, tp;
	int res;
	int ln=0;
	hotspot *hs;

	fptr = fopen(filename, "r");

	while (!feof(fptr))
	{
		if (!fgets(linebuf, 2048, fptr)) { break; }	
	
		if (linebuf[0] == '#') { continue; } // commment
		
		res = sscanf(linebuf, "%d,%d,%d,%d,%d,%[^\n]\n",&tp,&x1,&y1,&x2,&y2,cmdbuf);
		if (res<5) {
			fprintf(stderr,"parse error at line %d\n", ln);
			break;
		}

		hs = hotspot_create(tp, x1, y1, x2, y2, cmdbuf);
		if (hs==NULL) { return -1; }
		hotspot_add(hs);
		
		ln++;
	}
	return ln;
}

hotspot *hotspot_getHead(void)
{
	return g_hotspots;
}


