/*
laserspotcam        A software to track a laser pointer on a wall.
Copyright (C) 2005  Raphael Assenat

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author: Raphael Assenat <raph@raphnet.net>
*/
#ifndef _webcam_h
#define _webcam_h

/* Initialization*/
int initialise_webcam(char *device_file);
int close_webcam();

/* Change options */
/*
	printf("\nGet picture properties:\n");
	printf("Brightness: %d\n",videopic.brightness);
	printf("       Hue: %d\n",videopic.hue);
	printf("    Colour: %d\n",videopic.colour);
	printf("  Contrast: %d\n",videopic.contrast);
	printf(" Whiteness: %d\n",videopic.whiteness);
	printf("     Depth: %d\n",videopic.depth);
	printf("   Palette: %d\n",videopic.palette);*/

unsigned short set_webcam_brightness(unsigned short brightness);
unsigned short get_webcam_brightness();
unsigned short set_webcam_contrast(unsigned short contrast);
unsigned short get_webcam_contrast();
unsigned short set_webcam_gamma_correction(unsigned short gamma);
unsigned short get_webcam_gamma_correction();
unsigned short set_webcam_colour_saturation(unsigned short saturation);
unsigned short get_webcam_colour_saturation();

/* Return true if the webcam is a philips webcam */
char is_webcam_philips();
int get_webcam_name(char *name, int len);

/* Set the compression preference. 0 = Not compressed, 3 = Maximum compression */
int set_webcam_compression_preference(int level);

/* Get the compression preference. 0 = Not compressed, 3 = Maximum compression */
int get_webcam_compression_preference(void);

/* Reset all internal webcam settings to factory default */
void restore_webcam_factory_default(void);

/* Write current settings in the EEPROM */
void store_settings_in_webcam(void);

/* Restore settings from the EEPROM */
void restore_settings_from_webcam(void);

/* The on and off time in milliseconds */
int set_webcam_led_cycle(int ms_on, int ms_off);

/* Between 0 and 65535. Any negative number will enable automatic shutter speed */
int set_webcam_shutter_speed(int speed);


/* Between 0 and 65535. give a negative number to enable agc. */
int set_webcam_agc(int agc);
int get_webcam_agc();

/* 0: Auto  1: Manual  2: Indoor  3: Outdoor  4: Fluorescent */
int set_webcam_white_balance_mode(int mode);
int get_webcam_white_balance_mode();
int get_webcam_white_balance_values(int *r, int *b);

/* Any rate between 5 and 30 */
int set_webcam_capture_rate(char rate);
int get_webcam_capture_rate();

/* Any resolution supported by the webcam */
int set_webcam_resolution(int w, int h);
int get_webcam_resolution(int *w, int *h);

/* Get the file descriptor (to read or mmap) */
int get_webcam_filedescriptor();

#endif
